import os, sys, re, pickle

import numpy as np
import pandas as pd
import tensorflow as tf
from tensorflow.keras import models, layers
import matplotlib.pyplot as plt
from sklearn.metrics import classification_report
from sklearn.metrics import confusion_matrix
from glob import glob
import itertools

#Directories
fypdir = '/mnt/lustre/users/ebailey/fyp/'
datadir = '/mnt/lustre/users/ebailey/fyp/data/'
mri_datadir = '/mnt/lustre/users/ebailey/fyp/mri_datadir/'
resultsdir = '/mnt/lustre/users/ebailey/fyp/results/'

#Load pre-saved X and Y input arrays
X = np.load(datadir + 'all_MRIs_50%.npy')
X = np.expand_dims(X, axis=-1)    

Y = np.load(datadir + 'all_labels.npy')

#Split input data into train and test data
train_index = int(0.8*len(X))
X_train, X_test = X[:train_index], X[train_index:]
y_train, y_test = Y[:train_index], Y[train_index:]

#CNN Architecture
tf.keras.backend.clear_session()

inputs = layers.Input(shape=X_train.shape[1:])
from tensorflow.keras.layers import LeakyReLU
#LeNet-5 Architecture
#Convolutional 1
x = layers.Conv3D(filters=32, kernel_size=(3,3,3), strides=2, padding = 'same', activation=LeakyReLU(alpha=0.1))(inputs)
#Pooling 1
x = layers.MaxPooling3D(pool_size=(2,2,2))(x)
#Dropout
x = layers.Dropout(0.5)(x)

#Convolutional 2
x = layers.Conv3D(filters=64, kernel_size=(3,3,3), strides=2, padding = 'same', activation=LeakyReLU(alpha=0.1))(x)
#Pooling 2
x = layers.MaxPooling3D(pool_size=(2,2,2))(x)
#Dropout
x = layers.Dropout(0.5)(x)

x = layers.Flatten()(x)

#Fully connected 1
x = layers.Dense(120, activation='relu')(x)
#Fully connected 2
x = layers.Dense(84, activation='relu')(x)
#Fully connected 3
x = layers.Dense(1, activation = 'sigmoid')(x)

model = models.Model(inputs=inputs, outputs=x)
from tensorflow.keras.optimizers import Adam
#Allows you to adjust the learning rate
adam = Adam(lr=0.00001)
model.compile(loss='binary_crossentropy', optimizer=adam, metrics=['accuracy'])

#Train the model
n_epochs = 200
history = model.fit(X_train, y_train, epochs=n_epochs, \
                    batch_size=32, validation_data=(X_test, y_test), verbose=1)

#Model Performance
hist = history.history
print('max accuracy')
print(max(hist['val_acc']))
print('model evaluate')
print(model.evaluate(X_test, y_test))
with open(resultsdir + 'history.pickle', 'wb') as f:
    pickle.dump(hist, f)

y_pred = (model.predict(X_test) > 0.5).astype(int)
print(classification_report(y_test, y_pred, target_names=['patient','control']))

def plot_confusion_matrix(cm, classes, normalize=True, title='Confusion matrix', cmap=plt.cm.Blues):
    if normalize:
       cm = cm.astype('float') / cm.sum(axis=1)[:, np.newaxis]

    fig = plt.figure()
    plt.imshow(cm, interpolation='nearest', cmap=cmap)
    plt.title(title)
    plt.colorbar()
    tick_marks = np.arange(len(classes))
    plt.xticks(tick_marks, classes, rotation=45)
    plt.yticks(tick_marks, classes)
 
    fmt = '.2f' if normalize else 'd'
    thresh = cm.max() / 1.5 if normalize else cm.max() / 2
    for i, j in itertools.product(range(cm.shape[0]), range(cm.shape[1])):
        plt.text(j, i, format(cm[i, j], fmt), horizontalalignment='center', color='white' if cm[i, j] > thresh else 'black')
 
    plt.tight_layout()
    plt.ylabel('True label')
    plt.xlabel('Predicted label')

    fig.savefig(resultsdir  + 'confusion.png')

categorical_test_labels = pd.DataFrame(y_test)
categorical_preds = pd.DataFrame(y_pred)
confusion_matrix = confusion_matrix(categorical_test_labels, categorical_preds)
#Plot confusion matrix as a colour coded matrix 
plot_confusion_matrix(confusion_matrix, ['patient', 'control'], normalize=True)
