import os, sys, re, pickle
import numpy as np
from glob import glob

#Directories
fypdir = '/mnt/lustre/users/ebailey/fyp/'
datadir = '/mnt/lustre/users/ebailey/fyp/data'
mri_datadir = '/mnt/lustre/users/ebailey/fyp/mri_datadir/'
resultsdir = '/mnt/lustre/users/ebailey/fyp/results'

def parse_folder_for_MRIs(mri_datadir):
    #Find the desired files in each subject folder
    found_files = glob(os.path.join(mri_datadir,'**/anatomy/*brain.nii') )
    if len(found_files)==0:
        print('Did not find any nifti files in ', mri_datadir)
    else:
        size_in_MB = []
        for file in found_files:
            size_in_MB.append(os.path.getsize(file)/1024/1024)

        print('Number of nifti files', len(size_in_MB))
        print('Mean file size {:.2f} MB'.format(sum(size_in_MB)/len(size_in_MB)) )
        print('Total size {:.2f} GB'.format(sum(size_in_MB)/1024) )

    return found_files


def save_MRIs_as_ndarray(files, subIDs, datadir, dtype='float16'):
    import nibabel
    import nibabel.processing

    # Save subject IDs in pickle
    with open(datadir + 'subjectIDs.pickle', 'wb') as f:
        pickle.dump(subIDs, f)

    # Get dtype
    shape = nibabel.load(files[0]).header.get_data_shape()
    im = np.zeros( (len(files),) + shape, dtype=dtype)
    print('Shape of MRI ndarray', im.shape)


    # Load nifti files and add into array
    fname = os.path.join(datadir + 'all_MRIs_' + dtype)
    for ix, file in enumerate(files):
        ni = nibabel.load(file)
        im[ix,:,:,:] = ni.get_fdata()
    im = im.astype(dtype)

    #Save X array
    print('Saving as', fname)
    np.save(fname, im)

def extract_subjects_from_filenames(files):
    #Split the filename to extract the corresponding subjectID
    IDs = []
    for file in files:
        splitFile = file.split('_2')
        splitFile = splitFile[0].split('/w')
        subID = splitFile[1]
        IDs.append(subID)

    return IDs

def load_csv(fypdir):
    df = pd.read_csv(os.path.join(fypdir, 'demographics.csv'))
    return df

def get_subject_status(subIDs, df):
    #Find each subject in the csv file and retrieve their status (patient or control)
    n = len(subIDs)
    status = np.zeros((n, 1))
    for ix, s in enumerate(subIDs):
        s = s.lower().replace('-',"")
        s = s.lower().replace('_',"")
        df_sub = df[df['SubjectID'].str.replace('-',"")==s]
        status[ix] = df_sub['Patient_Control']
    return status


files = parse_folder_for_MRIs(mri_datadir)
subIDs = extract_subjects_from_filenames(files)
save_MRIs_as_ndarray(files, subIDs, datadir)
df = load_csv(fypdir)
Y = get_subject_status(subIDs, df)

#Save Y array
fname = os.path.join(datadir + 'all_labels')
np.save(fname, Y)